﻿;****************************************************************************
; CP/M 3.0 BOOT LOADER MODULE FOR THE S100Computers Z80-SBC Board
;
;	WRITTEN BY 		JOHN MONAHAN  
;	MODIFIED BY		ANDREW BINGHAM 04/07/2018
;
;****************************************************************************

;	04/07/2018	ZSOS v0.20	Updated console routines to respect
;					IOBYTE switches on Z80 SBC
;					General cleanup of tabs/code formatting


TITLE	'BOOT LOADER MODULE FOR CP/M 3.0'

	; DEFINE LOGICAL VALUES:
TRUE	EQU	-1
FALSE	EQU	NOT TRUE

	; DETERMINE IF FOR BANK SELECT OR NOT:
	; Note use spaces here instead of tabs so that build scripts can
	; flip this back and forth
BANKED EQU FALSE

	; DEFINE PUBLIC LABELS:
PUBLIC	?INIT,?LDCCP,?RLCCP,?TIME
PUBLIC	OUT$BLOCKS

	; EXTERNALLY DEFINED ENTRY POINTS AND LABELS:
EXTRN	?PMSG,?CONIN
EXTRN	@CIVEC,@COVEC,@AIVEC,@AOVEC,@LOVEC
EXTRN 	@CBNK,?BNKSL

IF	BANKED
	EXTRN	BANKBUF		;128 BYTE BUFFER IN MOVE MODULE FOR USE
				; DURING COLD AND WARM BOOTS
ENDIF

EXTRN	@SEC,@MIN,@HOUR,@DATE	;FIELDS HOLDING CURRENT TIME AND DATE


	; INCLUDE Z-80 MACROS:
MACLIB	Z80

	; SOME MISCELLANEOUS EQUATES:
BDOS	EQU	5
CR	EQU	13		;ASCII CARRIAGE RETURN
LF	EQU	10		;ASCII LINEFEED

; Note on the mapping below: The DIP switches on the physical board connect to GND with pull up resistors 
; OPEN/OFF switches are '1's and CLOSED/ON DIP switches are '0's
;
; BIT MAP OF IOBYTE BASE_PORT + 6H:- X X X X  X X X X  (if xx0xxx11= CONFIG, will use onboard USB chip for Console I/O)
;			             | | | |  | | | |..............  0=CONSOLE IN DATA from Console IO board
;			             | | | |  | | |................  0=CONSOLE OUT DATA to Console IO board
;			             | | | |  | |..................  0=CONSOLE OUT DATA also to Printer (unused)
;			             | | | |  |....................  Unused
;			             | | | |.......................  Unused			
;			             | | |.........................  1=IOBYTE not active/implemented (output to Console IO Board)							 
;			             | |...........................  0=data CAN be written to USB (TXE status)
;			     	     |.............................  0=data available to recieve from USB (RXE status)
								   
BASEPORT	EQU 	30H
IOBYTE 		EQU 	BASEPORT+6H 	;See above
IOB$NOT$ACTIVE	EQU	20H
CON$IN$PROP	EQU	01H
CON$OUT$PROP	EQU	02H


	; WE CAN DO INITIALIZATION FROM BANKED MEMORY (IF WE HAVE IT):
IF	BANKED
	DSEG			; INIT DONE FROM BANKED MEMORY
ELSE
	CSEG			; INIT TO BE DONE FROM COMMON MEMORY
ENDIF	


	;;;;; ?INIT
	; HARDWARE INITIALIZATION OTHER THAN CHARACTER AND DISK I/O:
?INIT:
	; SET EVERYTHING UP FOR USB BY DEFAULT
	; ASSIGN CONSOLE TO PROPELLER
	LXI	H,4000H		;SIGNIFIES DEVICE 1 (PROPELLER)
	SHLD	@CIVEC		;CONSOLE INPUT VECTOR
	SHLD	@COVEC		;CONSOLE OUTPUT VECTOR
	; ASSIGN AUX TO USB
	LXI	H,8000H		;SIGNIFIES DEVICE 0 (USB)
	SHLD	@AIVEC		;AUXILLIARY INPUT VECTOR
	SHLD	@AOVEC		;AUXILLIARY OUTPUT VECTOR
	; ASSIGN PRINTER TO LPT:
	LXI	H,2000H		;SIGNIFIES DEVICE 2 (N/A)
	SHLD	@LOVEC		;LIST OUTPUT VECTOR
	
	IN	IOBYTE		;READ IN IOBYTE
	ANI	IOB$NOT$ACTIVE	;IS IOBYTE EVEN ACTIVE?
	JNZ	CINITDONE	;IF NOT ACTIVE, THEN WE ARE DONE, DEFAULT TO PROPELLER BOARD	

	IN	IOBYTE		;READ IN IOBYTE
	ANI	CON$IN$PROP	;PROP FOR CONSOLE IN? 
	JZ 	NEXTTEST	;IF 0, LEAVE PROP FOR CONSOLE IN
	LXI	H,8000H		;SIGNIFIES DEVICE 0 (USB)
	SHLD	@CIVEC		;CONSOLE INPUT VECTOR
	LXI	H,4000H		;SIGNIFIES DEVICE 1 (PROPELLER)
	SHLD	@AIVEC		;AUXILLIARY INPUT VECTOR

NEXTTEST:	
	IN	IOBYTE		;READ IN IOBYTE
	ANI	CON$OUT$PROP	;PROP FOR CONSOLE OUT?
	JZ 	CINITDONE	;IF 0, LEAVE PROP FOR CONSOLE OUT
	LXI	H,8000H		;SIGNIFIES DEVICE 0 (USB)
	SHLD	@COVEC		;CONSOLE OUTPUT VECTOR
	LXI	H,4000H		;SIGNIFIES DEVICE 1 (PROPELLER)
	SHLD	@AOVEC		;AUXILLIARY OUTPUT VECTOR

	; PRINT THE SIGN-ON MESSAGE:
CINITDONE:
	LXI	H,SIGNON$MSG	;POINT TO IT
	JMP	?PMSG		;AND PRINT IT

	;;;;; OUT$BLOCKS
	; ROUTINE OUTPUTS SPECIFIED # BYTES TO SPECIFIED OUTPUT PORTS:
IF	BANKED
	CSEG			;WE WANT THIS ROUTINE IN COMMON MEMORY
ENDIF
OUT$BLOCKS:
	MOV	A,M		;GET A BYTE FROM THE BLOCK
	ORA	A		;END OF OUTPUT BLOCK ?
	RZ			;THEN DONE!!
	MOV	B,A		;ELSE PUT # BYTES TO SEND OUT IN [B]
	INX	H		;POINT TO PORT TO SEND TO
	MOV	C,M		;GET IT TO [C]
	INX	H		;POINT TO 1ST BYTE OF BLOCK TO SEND OUT
	OUTIR			;Z-80 BLOCK OUTPUT
	JR	OUT$BLOCKS


	;;;;; ?LDCCP
	; THIS ROUTINE IS ENTERED TO LOAD THE CCP.COM FILE INTO THE TPA BANK
	;  AT SYSTEM COLD START:
?LDCCP:
	; SET UP THE FCB FOR THE FILE OPERATION:
	XRA	A		;ZERO EXTENT
	STA	CCP$FCB+15
	LXI	H,0		;START AT BEGINNING OF FILE
	SHLD	FCB$NR

	; TRY TO OPEN THE CCP.COM FILE:
	LXI	D,CCP$FCB	;POINT TO FCB
	CALL	OPEN		;ATTEMPT THE OPEN OPERATION
	INR	A		;WAS IT ON THE DISK ?
	JRNZ	CCP$FOUND	;YES -- GO LOAD IT

	; WE ARRIVE HERE WHEN CCP.COM FILE WASN'T FOUND:
	LXI	H,CCP$MSG	;REPORT THE ERROR
	CALL	?PMSG
	CALL	?CONIN		;GET A RESPONSE
	JR	?LDCCP		;AND TRY AGAIN

	; FILE WAS OPENED OK -- READ IT IN:
CCP$FOUND:
	LXI	D,0100H		;LOAD AT BOTTOM OF TPA
	CALL	SETDMA		;BY SETTING THE NEXT DMA ADDRESS
	LXI	D,128		;SET MULTI SECTOR I/O COUNT
	CALL	SETMULTI	; TO ALLOW UP TO 16K BYTES IN ONE OPERATION
	LXI	D,CCP$FCB	;POINT TO THE FCB
	CALL	READ		;AND READ THE CCP IN
	;; NO BANK 2 ON THE Z80 SBC, SO REMOVED IF STATEMENT AND TAKE SAME
	;; ACTION AS UNBANKED VERSION
	RET



	;;;;; ?RLCCP
	; NO BANK 2 ON Z80 SBC, RELOAD FROM DISK REGARDLESS OF BANKED OR UNBANKED
?RLCCP:
	JMP	?LDCCP			;JUST DO LOAD AS THOUGH COLD BOOT


	;;;;; ?TIME
	; ROUTINE SETS/GETS TIME:
?TIME:	
	RET				;Just return in this simple version


IF	BANKED
	CSEG
ENDIF

	;;;;;
	; CP/M BDOS FUNCTION INTERFACES

	; OPEN FILE:
OPEN:
	MVI C,15 ! JMP BDOS		; OPEN FILE CONTROL BLOCK

	; SET DMA ADDRESS:
SETDMA:
	MVI C,26 ! JMP BDOS		; SET DATA TRANSFER ADDRESS

	; SET MULTI SECTOR I/O COUNT:
SETMULTI:
	MVI C,44 ! JMP BDOS		; SET RECORD COUNT

	; READ FILE RECORD:
READ:
	MVI C,20 ! JMP BDOS		; READ RECORDS

	; CCP NOT FOUND ERROR MESSAGE:
CCP$MSG:
	DB	CR,LF,'BIOS ERR ON A: NO CCP.COM FILE',0


	; FCB FOR CCP.COM FILE LOADING:
CCP$FCB:
	DB	1			;AUTO-SELECT DRIVE A
	DB	'CCP     COM'		;FILE NAME AND TYPE
	DB	0,0,0,0
	DS	16
FCB$NR:	DB	0,0,0


	; SYSTEM SIGN-ON MESSAGE:
SIGNON$MSG:
	DB	CR,LF,LF,'ZSOS v0.30 - a distribution of CP/M 3.0 '
	IF BANKED
		DB '(BANKED)'
	ELSE
		DB '(NON-BANKED)'
	ENDIF
	DB	CR,LF,'Image by A. Bingham, 28 May 2018'
	DB	CR,LF,'Original code by J. Monahan 16 March 2011'
	DB	CR,LF,'No-holes IDE/CF LBA routines by D. Fry'
	DB	CR,LF,'A:=IDE Hard Disk/CF Card. (No Floppy Disks!)'
	DB	CR,LF,0
	END
